/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __GPIO_NAND_H
#define __GPIO_NAND_H

/* Includes ------------------------------------------------------------------*/
#include <NUC1xx.H> 
#include "Driver\DrvSYS.h"

/* Exported types ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
//K9F2G08U0C SCB0 Series Device ID = EC DA 10 95 44
//NANDFLASH ID = 0xec da 10 95 44 	//read from chip by code!!!
typedef struct
{
  uint8_t Maker_ID;
  uint8_t Device_ID;
  uint8_t Third_ID;
  uint8_t Fourth_ID;
  uint8_t Fifth_ID;
}NAND_IDTypeDef;

typedef struct 
{
  uint16_t Plane;
  uint16_t Block;
  uint16_t Page;
} EBI_NAND_ADDRESS;

/* Exported constants --------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* NAND Area definition  for K9F2G08U0C Series @ NUC122RD2AN */
#define NF_RB     		   		    E_GPC, 0 		/* PC0  = R/B#  */
#define NF_RST  				    E_GPC, 1 		/* PC1  = RST#  */
#define NF_ALE				   	    E_GPA, 10 		/* PA10 = ALE   */
#define NF_CLE				   	    E_GPA, 11 		/* PA11 = CLE   */
#define NF_WEN				   	    E_GPA, 12 		/* PA12 = WE#   */
#define NF_WPN				   	    E_GPA, 13 		/* PA13 = WP#   */
#define NF_CEN				   	    E_GPA, 14 		/* PA14 = CE#   */
#define NF_REN				   	    E_GPA, 15 		/* PA15 = RE#   */

#define NF_CLE_HIGH 				(GPIOA->DOUT |=  (1 << 10))
#define NF_CLE_LOW	 				(GPIOA->DOUT &= ~(1 << 10))
#define NF_ALE_HIGH 				(GPIOA->DOUT |=  (1 << 11))
#define NF_ALE_LOW	 				(GPIOA->DOUT &= ~(1 << 11))
#define NF_WEN_HIGH 				(GPIOA->DOUT |=  (1 << 12))
#define NF_WEN_LOW	 				(GPIOA->DOUT &= ~(1 << 12))
#define NF_CEN_HIGH 				(GPIOA->DOUT |=  (1 << 14))
#define NF_CEN_LOW	 				(GPIOA->DOUT &= ~(1 << 14))
#define NF_REN_HIGH 				(GPIOA->DOUT |=  (1 << 15))
#define NF_REN_LOW	 				(GPIOA->DOUT &= ~(1 << 15))

#define NF_DATA_LB	 				GPIOD
#define NF_DATA_HB	 				GPIOB
#define NF_IO0     		   		    E_GPD, 0 		/* PD0  = I/O0  */
#define NF_IO1  				    E_GPD, 1 		/* PD1  = I/O1  */
#define NF_IO2				   	    E_GPD, 2 		/* PD2  = I/O2  */
#define NF_IO3				   	    E_GPD, 3 		/* PD3  = I/O3  */
#define NF_IO4				   	    E_GPB, 4 		/* PB4  = I/O4  */
#define NF_IO5				   	    E_GPB, 5 		/* PB5  = I/O5  */
#define NF_IO6				   	    E_GPB, 6 		/* PB6  = I/O6  */
#define NF_IO7				   	    E_GPB, 7 		/* PB7  = I/O7  */
#define NF_DELAY				   	1

/* GPIO NAND memory command */
#define	NAND_CMD_READ1             ((uint8_t)0x00)
#define NAND_CMD_READ2	           ((uint8_t)0x30)

#define NAND_CMD_WRITE0            ((uint8_t)0x80)
#define NAND_CMD_WRITE1            ((uint8_t)0x10)

#define NAND_CMD_ERASE0            ((uint8_t)0x60)
#define NAND_CMD_ERASE1            ((uint8_t)0xD0)  

#define NAND_CMD_READID            ((uint8_t)0x90)
#define NAND_CMD_IDADDR            ((uint8_t)0x00)
	
#define NAND_CMD_STATUS            ((uint8_t)0x70)
#define NAND_CMD_RESET             ((uint8_t)0xFF)

/* NAND memory status */
#define NAND_VALID_ADDRESS         ((uint32_t)0x00000100)
#define NAND_INVALID_ADDRESS       ((uint32_t)0x00000200)
#define NAND_TIMEOUT_ERROR         ((uint32_t)0x00000400)
#define NAND_BUSY                  ((uint32_t)0x00000000)
#define NAND_ERROR                 ((uint32_t)0x00000001)
#define NAND_READY                 ((uint32_t)0x00000040)

/* GPIO NAND memory K9F2G08U0C Series parameters */
#define NAND_PAGE_SIZE             ((uint16_t)0x0800) /* 2048 bytes per page w/o Spare Area */
#define NAND_BLOCK_SIZE            ((uint16_t)0x0040) /* 64 x 2048 bytes pages per block */
#define NAND_DEVICE_SIZE           ((uint16_t)0x0800) /* 2048 Blocks per Device */
#define NAND_SPARE_AREA_SIZE       ((uint16_t)0x0040) /* last 64 bytes as spare area */

/* GPIO NAND memory K9F2G08U0C Series address computation */
#define ADDR_1st_CYCLE(PAGEADDR)   (uint8_t)(0x0)       			/* 1st addressing cycle */
#define ADDR_2nd_CYCLE(PAGEADDR)   (uint8_t)(0x0)					/* 2nd addressing cycle */
#define ADDR_3rd_CYCLE(PAGEADDR)   (uint8_t)((PAGEADDR>>0 ) & 0xFF)	/* 3rd addressing cycle */
#define ADDR_4th_CYCLE(PAGEADDR)   (uint8_t)((PAGEADDR>>8 ) & 0xFF)	/* 4th addressing cycle */
#define ADDR_5th_CYCLE(PAGEADDR)   (uint8_t)((PAGEADDR>>16) & 0x01)	/* 5th addressing cycle */
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

uint32_t GPIO_NAND_GetStatus(void);
uint32_t GPIO_NAND_ReadStatus(void);
/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */
extern void GPIO_NAND_Init(void);
extern uint32_t GPIO_NAND_Reset(void);
extern uint32_t GPIO_NAND_ReadID(NAND_IDTypeDef* NAND_ID);
extern uint32_t GPIO_NAND_EraseBlock(uint32_t BlockAddress);
extern uint32_t GPIO_NAND_WritePage_test(uint8_t data, uint32_t PageAddress);
extern uint32_t GPIO_NAND_ReadPage_test(uint32_t PageAddress);
#endif /* __GPIO_NAND_H */
